@@ -1,7 +1,8 @@
 import csv
 import pickle
 import os
-from azureml.core.compute import AmlCompute, ComputeTarget
+from azureml.core.compute import AmlCompute, ComputeTarget, RemoteCompute
+from azureml.core.compute_target import ComputeTargetException
 from azureml.core.datastore import Datastore
 from msrest.exceptions import HttpOperationError
 from azureml.core import Environment
@@ -261,7 +262,7 @@ class FileNameProvider:
 
 class ComputeTargetConfig:
     @staticmethod
-    def config(ws,cluster_name, vm_type, min_nodes, max_nodes,idle_seconds ):
+    def config_create(ws,cluster_name, vm_type, min_nodes, max_nodes,idle_seconds ):
         #Create or Attach existing compute resource
         # choose a name for your cluster
         compute_name = os.environ.get("AML_COMPUTE_CLUSTER_NAME", cluster_name)
@@ -294,6 +295,29 @@ class ComputeTargetConfig:
         
         return compute_target
 
+    @staticmethod
+    def config_attach(ws,compute_target_name, resource_id,username, password):
+        try:
+            attached_dsvm_compute = RemoteCompute(workspace=ws, name=compute_target_name)
+            print('found existing:', attached_dsvm_compute.name)
+        except ComputeTargetException:
+            # Attaching a virtual machine using the public IP address of the VM is no longer supported.
+            # Instead, use resourceId of the VM.
+            # The resourceId of the VM can be constructed using the following string format:
+            # /subscriptions/<subscription_id>/resourceGroups/<resource_group>/providers/Microsoft.Compute/virtualMachines/<vm_name>.
+            # You can also use subscription_id, resource_group and vm_name without constructing resourceId.
+            
+            attach_config = RemoteCompute.attach_configuration(resource_id=resource_id,
+                                                                ssh_port=22,
+                                                                username=username,
+                                                                password=password)
+            attached_dsvm_compute = ComputeTarget.attach(ws, compute_target_name.strip(), attach_config)
+            attached_dsvm_compute.wait_for_completion(show_output=True)
+
+        return attached_dsvm_compute
+        
+
+
 class DataStoreConfig:
     @staticmethod
     def config(ws, blob_datastore_name,account_name,container_name,account_key):
@@ -958,6 +982,34 @@ class LazyDataCorresponder:
                 pass
         return X_data, y_data, id_data
 
+
+class SplittedDataCorresponder:
+    def __init__(self, all_vec_data_directory_path):
+        self.all_vec_data_directory_path = all_vec_data_directory_path
+    
+    def get_vec_correspondence(self, data_text):
+        if not data_text :
+            raise Exception("data_text cannot be empty")
+
+        X_data, y_data, id_data  = [], [], []
+        for block_file in os.listdir(self.all_vec_data_directory_path):
+            result = joblib.load(os.path.join(self.all_vec_data_directory_path, block_file))
+            vec_result, _, id_result = result
+            id_result = [item.decode("utf-8") for item in id_result]
+            #print(id_result)
+            #id_result, target_result = id_result.decode("utf-8"), target_result.decode("utf-8")
+            for item in data_text:
+                id = item[0]
+                label = item[2]
+                #print("{0} == {1}".format(id, id_result))
+                #print("{0} == {1}".format(type(id), type(id_result)))
+                if id in id_result:
+                    X_data.append(vec_result)
+                    y_data.append(label)
+                    id_data.append(int(id))
+            result = None #freed memory
+        return X_data, y_data, id_data
+
 class DataAdapter:
     def __init__(self):
         self.featuresHandler = FeaturesHandler()
@@ -1314,3 +1366,55 @@ class HumlLargeFileWriter:
         for _thread in threadlist:
             _thread.join()
 
+
+class HumlLargeFileUploader:
+    def __init__(self,directory_path, blob_storage_manager, featuresHandler, feature_index):
+        self.directory_path = directory_path
+        self.blob_storage_manager = blob_storage_manager
+        self.featuresHandler = featuresHandler
+        self.feature_index = feature_index
+    
+    def _upload_file(self, data, file_blob_dest, index, ext):
+        filename = "block_{0}.{1}".format(index, ext)
+        file = os.path.join(self.directory_path, filename)
+        #if not os.path.isfile(file):
+        #self._log("{0} beggin".format(file)) 
+        #data = X,y,ids
+        joblib.dump(data, file)
+        print("file {0} dumped".format(file))
+
+        #upload(self.VECTORIZED_UNLABELED_DATA_PATH, unlabeled_data_local_file, overwrite_v = True)
+        self.blob_storage_manager.upload(file_blob_dest, file, overwrite_v = True)
+
+        print("file {0} uploaded".format(file))
+        os.remove(file) #disk space is freed up here
+
+    def upload(self, file_source,file_blob_dest, ext, block_files_count):
+        X, y, ids = [],[],[]
+        index = 0
+        with open(file_source) as f:
+            content = f.readlines()
+        limite = len(content)//block_files_count
+        #print("limite :", limite)
+        #return 
+        with open(file_source, 'rb') as infile:
+            for line in infile:
+                items = line.split(b',')
+                _id = items[0]
+                features = items[1].split()
+                feature_vec = self.featuresHandler.make_feature_vector(features,self.feature_index)
+                target = ""
+                if len(items) > 2:
+                    target = items[2]
+
+                X.append(feature_vec)
+                y.append(target)
+                ids.append(_id)
+                if len(X) == limite:
+                    print("limite :", limite)
+                    self._upload_file((X,y,ids),file_blob_dest,index, ext)
+                    index += 1
+                    X, y, ids = [],[],[] # the memory is emptied here
+            
+            if len(X) > 0:
+                self._upload_file((X,y,ids), file_blob_dest, index, ext)
